local insetValue = 2;

CUI_FRAME_BACKGROUND = "Interface\\AddOns\\CleanUI\\skins\\frame\\frame-background";
CUI_FRAME_BACKGROUND_OPAQUE = "Interface\\AddOns\\CleanUI\\skins\\frame\\frame-background-opaque";
CUI_FRAME_BORDER = "Interface\\AddOns\\CleanUI\\skins\\frame\\frame-border";

function CleanUI_SetBackdrop(frame, addBackground, opaque, otherborder, otherbg, top, left, bottom, right)
    local border = otherborder or CUI_FRAME_BORDER;
    local bg = otherbg or CUI_FRAME_BACKGROUND;

    local parent = frame;

    if (top or left or bottom or right) then
        local backdropFrame = _G[frame:GetName().."BackdropFrame"];
        if (not backdropFrame) then
            backdropFrame = CreateFrame("Frame", frame:GetName().."BackdropFrame", frame);
            backdropFrame:SetParent(frame);
            backdropFrame:SetFrameStrata("BACKGROUND");
        end

        backdropFrame:ClearAllPoints();
        backdropFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", (left or 0), (top or 0));
        backdropFrame:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", (right or 0), (bottom or 0));

        parent = backdropFrame;
    end

    if (opaque) then
        bg = CUI_FRAME_BACKGROUND_OPAQUE;
    end

    if (addBackground == false) then
        parent:SetBackdrop( {
            edgeFile = border, tile = true, tileSize = 64, edgeSize = 16,
            insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
        });
    else
        parent:SetBackdrop( {
            bgFile = bg,
            edgeFile = border, tile = true, tileSize = 64, edgeSize = 16,
            insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
        });
    end
end

function CleanUI_SetBackdropWithoutBorder(frame, opaque)
    local bg = CUI_FRAME_BACKGROUND;

    if (opaque) then
        bg = CUI_FRAME_BACKGROUND_OPAQUE;
    end

    frame:SetBackdrop( {
        bgFile = bg, tile = true, tileSize = 64, edgeSize = 16,
        insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
    });
end

function CleanUI_GameMenuFrame_UpdateVisibleButtons()
    GameMenuFrame:SetSize(185, 370);
    GameMenuButtonLogout:ClearAllPoints();
    GameMenuButtonLogout:SetPoint("TOP", CleanUIGameMenuButtonReloadUI, "BOTTOM", 0, -10);
    
    if (not GameMenuFrame.info) then
        local title = GetAddOnMetadata("CleanUI", "Title");
        local version = GetAddOnMetadata("CleanUI", "Version");
    
        GameMenuFrame.info = GameMenuFrame:CreateFontString("GameMenuFrameInfo", "ARTWORK");
        GameMenuFrame.info:SetFontObject("CuiFont_ExtraSmall");
        GameMenuFrame.info:SetText("powered by |c333399ff"..title.."|r ("..version..")");
        GameMenuFrame.info:SetJustifyH("RIGHT");
        GameMenuFrame.info:SetSize(180, 15);
        GameMenuFrame.info:ClearAllPoints();
        GameMenuFrame.info:SetPoint("BOTTOMRIGHT", GameMenuFrame, "BOTTOMRIGHT", -10, 8);
    end
end

function CleanUI_UpdateAccountData()
    local guid = UnitGUID("player");
    
    local name = UnitName("player");
    
    CleanUIAccountData[guid] = CleanUIAccountData[guid] or {};
    
    CleanUIAccountData[guid].name = UnitName("player");
    CleanUIAccountData[guid].realm = GetRealmName();
    CleanUIAccountData[guid].class = UnitClass("player");
    CleanUIAccountData[guid].race = UnitRace("player");
    CleanUIAccountData[guid].faction = UnitFactionGroup("player");
    CleanUIAccountData[guid].level = UnitLevel("player");
end

function CleanUI_ResetMoney()
    CleanUICurrency["money"] = {};
end

function CleanUI_UpdateMoney()
    CleanUICurrency["money"] = CleanUICurrency["money"] or {};
    
    local money = CleanUICurrency["money"];
    
    money[UnitGUID("player")] = GetMoney();
end

-- Favorite mounts

local function setIsFavorite(spellID)
    
    cui_debug(spellID, "SET");
    local numMounts = C_MountJournal.GetNumMounts();
    for index = 1, numMounts do
        local _, foundSpellID = C_MountJournal.GetDisplayedMountInfo(index);
        
        if (spellID == foundSpellID) then
            C_MountJournal.SetIsFavorite(index, true);
            return;
        end
    end
    
    cui_debug(spellID, "NOT FOUND");
end

function CUI_setIsFavorite(spellID)
    setIsFavorite(spellID);
end

local function checkIsFavorite(spellID)
    local count = 0;
    
    local numMounts = C_MountJournal.GetNumMounts();
    for index = 1, numMounts do
        local _, foundSpellID, _, _, _, _, isFavorite = C_MountJournal.GetDisplayedMountInfo(index);
        if (spellID == foundSpellID and isFavorite) then
            return true;
        end
    end
    
    return false;
end

function CleanUI_LoadFavoriteMounts()
    CleanUserData.FavoriteMounts = CleanUserData.FavoriteMounts or {};
    
    if (#CleanUserData.FavoriteMounts == 0) then
        return;
    end
    
    -- remove all
    while C_MountJournal.GetIsFavorite(1) do
        C_MountJournal.SetIsFavorite(1, false);
    end
    
    local numFavorites = #CleanUserData.FavoriteMounts;
    
    for fav = 1, numFavorites do
        local spellID = CleanUserData.FavoriteMounts[fav];
        while (not checkIsFavorite(spellID)) do
            setIsFavorite(spellID);
        end
    end
end

function CleanUI_SaveFavoriteMounts()
    CleanUserData.FavoriteMounts = {};
    
    local numMounts = C_MountJournal.GetNumMounts();
    
    for index = 1, numMounts do
        local isFavorite = C_MountJournal.GetIsFavorite(index);
        
        if (isFavorite) then
            local _, spellID = C_MountJournal.GetDisplayedMountInfo(index);
            tinsert(CleanUserData.FavoriteMounts, spellID);
        end
    end
end

