local waitTable = {};
local waitFrame = nil;

local debug = true;

CUI_FRAME_BACKGROUND = "Interface\\AddOns\\CleanUI\\skins\\frame\\frame-background";
CUI_FRAME_BACKGROUND_OPAQUE = "Interface\\AddOns\\CleanUI\\skins\\frame\\frame-background-opaque";

-- raid icons
CUI_RAIDICON_FILE = "interface\\targetingframe\\ui-raidtargetingicons";

CUI_RAIDICONS = {};
CUI_RAIDICONS[0] = {left=0, right=0, top=0, bottom=0};
CUI_RAIDICONS[1] = {left=0, right=0.25, top=0, bottom=0.25};
CUI_RAIDICONS[2] = {left=0.25, right=0.5, top=0, bottom=0.25};
CUI_RAIDICONS[3] = {left=0.5, right=0.75, top=0, bottom=0.25};
CUI_RAIDICONS[4] = {left=0.75, right=1, top=0, bottom=0.25};
CUI_RAIDICONS[5] = {left=0, right=0.25, top=0.25, bottom=0.5};
CUI_RAIDICONS[6] = {left=0.25, right=0.5, top=0.25, bottom=0.5};
CUI_RAIDICONS[7] = {left=0.5, right=0.75, top=0.25, bottom=0.5};
CUI_RAIDICONS[8] = {left=0.75, right=1, top=0.25, bottom=0.5};
CUI_RAIDICONS[9] = {left=0, right=1, top=0, bottom=1};

-- functions
function cui_debug(value, name)
    if (not debug) then
        return;
    end

    if (name) then
        DEFAULT_CHAT_FRAME:AddMessage("debug "..name..": "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    else
        DEFAULT_CHAT_FRAME:AddMessage("debug: "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    end
end

-- functions
function cui_info(value, name)
    if (name) then
        DEFAULT_CHAT_FRAME:AddMessage("|c333399ff"..name..":|r "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    else
        DEFAULT_CHAT_FRAME:AddMessage("|c333399ffCleanUI:|r "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    end
end

function CleanUI_StartDelay(delay, func, ...)
    if(type(delay)~="number" or type(func)~="function") then
        return false;
    end

    if(waitFrame == nil) then
        waitFrame = CreateFrame("Frame","CUI_WaitFrame", UIParent);
        waitFrame:SetScript("onUpdate",function (self,elapse)
            local count = #waitTable;
            local i = 1;

            while(i<=count) do
                local waitRecord = tremove(waitTable,i);
                local d = tremove(waitRecord,1);
                local f = tremove(waitRecord,1);
                local p = tremove(waitRecord,1);

                if(d>elapse) then
                    tinsert(waitTable,i,{d-elapse,f,p});
                    i = i + 1;
                else
                    count = count - 1;
                    f(unpack(p));
                end
            end
        end);
    end

    tinsert(waitTable,{delay,func,{...}});

    return true;
end

function CleanUI_MakeFrameMoveable(frame)
    frame:SetMovable(true);
    frame:SetUserPlaced(true);
    frame:SetClampedToScreen(true);
    frame:SetClampRectInsets(1, -1, -1, 1);

    frame:HookScript("OnMouseDown", CleanUI_AnyFrameOnMouseDown);
    frame:HookScript("OnMouseUp", CleanUI_AnyFrameOnMouseUp);
end

function CleanUI_AnyFrameOnMouseDown(self, button)
    if (button == "LeftButton") then
        self:StartMoving();
    end
end

function CleanUI_AnyFrameOnMouseUp(self, button)
    self:StopMovingOrSizing();
end

function cui_rgb2hex(red, green, blue)

    if (not red) then
        red = 1.0;
    end

    if (not green) then
        green = 1.0;
    end

    if (not blue) then
        blue = 1.0;
    end

    local r = string.format("%x", red * 255);
    local g = string.format("%x", green * 255);
    local b = string.format("%x", blue * 255);

    if (strlen(r) < 2) then
        r = "0"..r;
    end
    if (strlen(g) < 2) then
        g = "0"..g;
    end
    if (strlen(b) < 2) then
        b = "0"..b;
    end

    local color = r..g..b;

    return color;
end

function CleanUI_RemoveUiButtonTexture(button)
    button.Left:Hide();
    button.Middle:Hide();
    button.Right:Hide();
end

function CleanUI_ReplaceAllTextures(texturename, frame, withTexture)
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and not child:IsForbidden() and child:IsObjectType("Frame")) then
            CleanUI_ReplaceTextureWith(texturename, child, withTexture);
            
            if (child:IsObjectType("frame")) then
                CleanUI_ReplaceAllTextures(texturename, child, withTexture);
            end
        end
    end
end

function CleanUI_ReplaceTextureWith(texturename, frame, withTexture)
    for i, child in ipairs({frame:GetRegions()}) do
        if (child and not child:IsForbidden() and child:IsObjectType("texture")) then
            local texture = child:GetTexture();
            
            if (texture) then
                if (string.find(texture, texturename, 1, true)) then
                    child:SetTexture(withTexture);
                end
            end
        end
    end
end

function CleanUI_RemoveTextures(frame)
    for i, child in ipairs({frame:GetRegions()}) do

        if (child and child:IsObjectType("texture")) then
            child:SetTexture(nil);
        end
    end
end

function CleanUI_GetRegionOfTypeAtPos(frame, type, pos)
    local index = 0;
    for i, child in ipairs({frame:GetRegions()}) do
        if (child and child:IsShown() and child:IsObjectType(type)) then
            if (index == pos) then
                return child;
            end
            index = index + 1;
        end
    end
end

function CleanUI_GetChildOfTypeAtPos(frame, type, pos)
    local index = 0;
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:IsShown() and child:IsObjectType(type)) then
            if (index == pos) then
                return child;
            end
            index = index + 1;
        end
    end
end

function CleanUI_GetChildWithName(frame, name)
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:GetName() == name) then
            return child;
        end
    end
end

function CleanUI_GetChildWithNameAtPos(frame, name, pos)
    local index = 0;
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:GetName() == name) then
            if (index == pos) then
                return child;
            end
            index = index + 1;
        end
    end
end

function CleanUI_ListChildren(frame)
    for i, child in ipairs({frame:GetRegions()}) do
        if (child) then
            cui_debug(child:GetName(), "region");
        end
    end
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:IsObjectType("Frame")) then
            cui_debug(child:GetName(), "frame");

            if (not child:IsObjectType("button")) then
                CleanUI_ListTextures(child);
            end
        end
    end
end

function CleanUI_ShowChildren(frame)
    for i, child in ipairs({frame:GetRegions()}) do
        if (child) then
            cui_debug(child:GetName(), "region");
            child:Show();
        end
    end
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:IsObjectType("Frame")) then
            cui_debug(child:GetName(), "frame");
            child:Show();

            if (not child:IsObjectType("button")) then
                CleanUI_ShowTextures(child);
            end
        end
    end
end

function CleanUI_ListTextures(frame)
    for i, child in ipairs({frame:GetRegions()}) do
        if (child and child:IsObjectType("texture")) then
            local texture = child:GetTexture();
            if (texture) then
                cui_debug(texture, child:GetName());
            end
        end
    end
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:IsObjectType("Frame")) then
            if (not child:IsObjectType("button")) then
                CleanUI_ListTextures(child);
            end
        end
    end
end

function CleanUI_ShowTextures(frame)
    for i, child in ipairs({frame:GetRegions()}) do
        if (child and child:IsObjectType("texture")) then
            child:Show();
            local texture = child:GetTexture();
            if (texture) then
                cui_debug(texture, child:GetName());
            end
        end
    end
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:IsObjectType("Frame")) then
            child:Show();
            if (not child:IsObjectType("button")) then
                CleanUI_ListTextures(child);
            end
        end
    end
end

function CleanUI_SetBackdrop(frame, addBackground, opaque, otherborder, otherbg, top, left, bottom, right)
    local border = otherborder or CleanUI_GetFrameBorder();
    local bg = otherbg or CUI_FRAME_BACKGROUND;

    local parent = frame;

    if (top or left or bottom or right) then
        local backdropFrame = _G[frame:GetName().."BackdropFrame"];
        if (not backdropFrame) then
            backdropFrame = CreateFrame("Frame", frame:GetName().."BackdropFrame", frame);
            backdropFrame:SetParent(frame);
            backdropFrame:SetFrameStrata("BACKGROUND");
        end

        backdropFrame:ClearAllPoints();
        backdropFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", (left or 0), (top or 0));
        backdropFrame:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", (right or 0), (bottom or 0));

        parent = backdropFrame;
    end

    if (opaque) then
        bg = CUI_FRAME_BACKGROUND_OPAQUE;
    end

    if (addBackground == false) then
        parent:SetBackdrop( {
            edgeFile = border, tile = true, tileSize = 64, edgeSize = 16,
            insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
        });
    else
        parent:SetBackdrop( {
            bgFile = bg,
            edgeFile = border, tile = true, tileSize = 64, edgeSize = 16,
            insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
        });
    end
end

function CleanUI_SetBackdropWithoutBorder(frame, opaque)
    local bg = CUI_FRAME_BACKGROUND;

    if (opaque) then
        bg = CUI_FRAME_BACKGROUND_OPAQUE;
    end

    frame:SetBackdrop( {
        bgFile = bg, tile = true, tileSize = 64, edgeSize = 16,
        insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
    });
end

function CleanUI_GC()
    local before = collectgarbage("count");
    collectgarbage();
    local after = collectgarbage("count");

    local freed = before - after;

    local msg = "collectgarbage:\n"..
        "memory freed: |c00FFFF66"..string.format("%.2f", freed).." kb|r.\n"..
        "current memory use: |c00FFFF66"..string.format("%.2f", after).." kb|r.\n";

    DEFAULT_CHAT_FRAME:AddMessage(msg, 1.0, 1.0, 1.0, 1, 10);
end

function CleanUI_WoWBuildInfo()
    local version, build, date, tocversion = GetBuildInfo();
    cui_debug(version, "version");
    cui_debug(build, "build");
    cui_debug(date, "date");
    cui_debug(tocversion, "tocversion");
end

-- hook
function CleanUI_HookChangeStatusBar(texture)
end

function CleanUI_XXXXX(message)
    cui_debug(message, "message");
end
